#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#include "benny.h"

void new_game (game *);
void new_level (game *);
void intro (void);
int map_menu (void);

void battle_zone (game *);
void view_place (game *);

void display_screen (char *);
void pause (void);
void show_stats (game *);
void show_score (game *);
void get_user_info (game *);
void help (void);
int getint (void);
int line_getchar (void);

#ifdef WWIV
void chain_read (game *, char *);
#endif

main (int argc, char *argv[])
{
  int choice;
  game *current = malloc (sizeof (game));
  if (current == NULL)
    {
      fprintf (stderr, "not enough memory!");
      exit (1);
    }

  /* endless game loop */
  while ('A' == 'A')
    {
      intro ();
      new_game (current);

#ifdef WWIV
      if (argc < 2)
	chain_read (current, DEFAULT_CHAIN);
      else if (!strcmp (argv[1], "local"))
	get_user_info (current);
      else
	chain_read (current, argv[1]);
#else
      get_user_info (current);
#endif
      help ();
      /* loop repeated each level */
      while (!(current->game_over))
	{
	  new_level (current);

	  /* loop repeated while scum are alive */
	  while ((current->scum > 0) && (!current->game_over))
	    {
	      int choice = map_menu ();
	      if (choice == 'q')
		current->game_over = 1;
	      else if (current->losses[choice])
		{
		  printf ("That has been taken by the enemy\n");
		  printf ("Choose another location!\n");
		  pause ();
		  continue;
		}
	      else
		{
		  current->location = choice;
		  battle_zone (current);
		}
	    }
	}
      printf ("Play again (y/n)?\n");
      while (1)
	{
	  putchar ('>');
	  choice = tolower (line_getchar ());
	  if (choice == 'y' || choice == 'n')
	    break;
	}
      if (choice == 'n')
	break;
    }
  show_score (current);
  free (current);
  return 0;
}

void 
intro (void)
{
  display_screen ("bkgame.ans");
}

void 
new_game (game * g)
{
  g->level = 1;
  g->score = 0;
  g->game_over = 0;
}

void 
new_level (game * g)
{
  int i;
  for (i = 0; i < 10; i++)
    g->losses[i] = 0;
  g->forces = FORCES_START + FORCES_PER_LEVEL * (g->level - 1);
  g->scum = SCUM_START + SCUM_PER_LEVEL * (g->level - 1);
  g->retreats = 0;
  if (g->level < 3)
    strcpy (g->rank, "Lieutenant");
  else if (g->level < 5)
    strcpy (g->rank, "Captain");
  else if (g->level < 7)
    strcpy (g->rank, "Major");
  else if (g->level < 9)
    strcpy (g->rank, "Colonel");
  else
    strcpy (g->rank, "General");
}

int 
map_menu (void)
{
  char ch;
  display_screen ("bk.map");
  while (1)
    {
      putchar ('>');
      ch = line_getchar ();
      if ((ch >= '0') && (ch <= '9'))
	return (int) (ch - '0');
      if ((ch == 'q') || (ch == 'Q'))
	return (int) 'q';
    }
}

#ifdef WWIV
void 
chain_read (game * current, char *filename)
{
  char dummy[STRING_SIZE];
  FILE *fp;

  if ((fp = fopen (filename, "r")) == NULL)
    {
      get_user_info (current);
      return;
    }

  /* read WWIV user number but don't do anything with it */
  fgets (dummy, STRING_SIZE, fp);

  /* read the line with the users WWIV username   */
  fgets (dummy, STRING_SIZE - 1, fp);

  sscanf (dummy, "%s", current->name);
}

#endif

void 
get_user_info (game * current)
{
  printf ("Harr! Enter your name, Soldier!\n>");
  fgets (current->name, STRING_SIZE - 1, stdin);
  current->name[strlen (current->name) - 1] = 0;
}

void 
view_place (game * current)
{
  char filename[16];
  sprintf (filename, "bk-%d.ans", current->location);
  display_screen (filename);
}

void 
display_screen (char *filename)
{
  unsigned int ch;
  FILE *fp;

  fp = fopen (filename, "r");
  if (fp == NULL)
    {
      fprintf (stderr, "can't find %s\n", filename);
      pause ();
    }

  while ((ch = fgetc (fp)) != EOF)
    putchar (ch);

  fclose (fp);
}

void 
pause (void)
{
  char dummy[STRING_SIZE];
  printf ("---press enter to continue---");
  fgets (dummy, STRING_SIZE - 1, stdin);
}

int 
getint (void)
{
  int num;
  char dummy[16];
  fgets (dummy, 15, stdin);
  sscanf (dummy, "%d", &num);
  return num;
}

int 
line_getchar (void)
{
  char ch;
  char dummy[16];
  fgets (dummy, 15, stdin);
  sscanf (dummy, "%c", &ch);
  return (int) ch;
}

void 
show_stats (game * current)
{
  printf ("********************************************************\n");
  printf ("You:   %d\t\tThem:   %d\n", current->forces, current->scum);
  printf ("Score: %d\t\tRetreats: %d\n", current->score, current->retreats);
  printf ("********************************************************\n");
}

void 
show_score (game * current)
{
  printf ("\n************************************\n");
  printf ("Stats for %s:\n", current->name);
  printf ("Score: %d\n", current->score);
  printf ("Level: %d\n", current->level);
  printf ("Rank: %s\n", current->rank);
  printf ("************************************\n");
}

void 
help (void)
{
  char ch;
  printf ("Do you need instructions (y/n)?\n");
  do
    {
      putchar ('>');
      ch = tolower (line_getchar ());
    }
  while (ch != 'y' && ch != 'n');
  if (ch == 'y')
    {
      display_screen (HELP_FILE);
      pause ();
    }
}


void 
battle_zone (game * current)
{
  int max_forces, max_scum;
  current->forces = FORCES_START + FORCES_PER_LEVEL * (current->level - 1);

  view_place (current);
  while (1)
    {
      int choice, attack, you_hit, they_hit, possible_kills;

      show_stats (current);
      printf ("You face %d of the worst scum in %s, %s %s.\n",
	      current->scum, TOWN, current->rank, current->name);
      printf ("You choices are: <R>etreat, <F>ight, <Q>uit\n");
      printf ("\nWhat are your orders, %s?\n", current->rank);

      while (1)
	{
	  choice = 0;
	  putchar ('>');
	  choice = tolower (line_getchar ());
	  if (choice == 'r' || choice == 'f' || choice == 'q')
	    break;
	}

      switch (choice)
	{
	case 'q':
	  current->game_over = 1;
	  return;

	case 'r':
	  if (current->retreats >= MAX_RETREATS)
	    {
	      printf ("You have already retreated too many times!");
	      printf ("This is a battle to the death!");
	    }
	  else
	    {
	      (current->retreats)++;
	      current->losses[current->location] = 1;
	      return;
	    }
	}

      /* default to fight */

      printf ("How many to attack?\n");
      while (1)
	{
	  putchar ('>');
	  attack = getint ();
	  if (attack > 0 && attack <= current->forces)
	    break;
	}

      srand (time (NULL));
      possible_kills = 1 + (attack < current->scum ? attack : current->scum);

      you_hit = rand () % possible_kills;

      they_hit = rand () % possible_kills;

      if (you_hit)
	printf ("You %d killed scum, %s %s!\n",
		you_hit, current->rank, current->name);
      else
	printf ("You missed them, %s %s.\n", current->rank, current->name);

      if (they_hit)
	printf ("The scum killed %d of your forces.\n", they_hit);

      current->scum -= you_hit;
      current->forces -= they_hit;
      current->score += you_hit * 100;

      if (current->scum <= 0)
	{
	  int bonus = (MAX_RETREATS - current->retreats - 1) * 50
	  + current->forces * 10;
	  current->score += bonus;
	  current->level++;
	  printf ("You have destroyed the army of scum!\n");
	  printf ("Your bonus for this level is %d\n", bonus);
	  printf ("Your score is now %d\n", current->score);
	  pause ();
	  return;
	}

      if (current->forces <= 0)
	{
	  printf ("\n**** GAME OVER ****\n");
	  printf ("The scum are victorious.\n\n");
	  current->game_over = 1;
	  pause ();
	  return;
	}
      printf ("\n");
    }
}
